package com.smppw.analysis.client;

import com.smppw.analysis.application.dto.info.*;
import com.smppw.analysis.application.service.info.FundInfoService;
import com.smppw.analysis.application.service.info.GlobalService;
import com.smppw.analysis.domain.dto.info.*;
import com.smppw.common.pojo.ResultVo;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @author wangzaijun
 * @date 2023/8/7 15:40
 * @description 基本信息api
 */
@RestController
@RequestMapping("/v1/api/fund")
public class FundApi {
    private final FundInfoService service;
    private final GlobalService globalService;

    public FundApi(FundInfoService service, GlobalService globalService) {
        this.service = service;
        this.globalService = globalService;
    }

    /**
     * 标的头部信息接口
     *
     * @param params 请求参数
     * @return /
     */
    @GetMapping("head-info")
    public ResultVo<? extends HeadInfoVO> headInfo(HeadInfoReq params) {
        return ResultVo.ok(this.globalService.headInfo(params));
    }

    /**
     * 统一设置接口，返回所有标的的统一设置，包括时段、净值频率等
     *
     * @param params /
     * @return /
     */
    @GetMapping("common-info")
    public ResultVo<CommonInfoVO> commonInfo(CommonInfoReq params) {
        return ResultVo.ok(this.globalService.commonInfo(params));
    }

    /**
     * 头部指标计算接口
     *
     * @param params 请求参数
     * @return /
     */
    @GetMapping("head-indicator")
    public ResultVo<HeadIndicatorVO> headIndicator(HeadIndicatorReq params) {
        return ResultVo.ok(this.globalService.headIndicator(params));
    }

    /**
     * 相似产品（私募基金）
     *
     * @param params /
     * @return /
     */
    @GetMapping("similar")
    public ResultVo<List<FundSimilarVO>> similar(FundSimilarReq params) {
        return ResultVo.ok(this.service.getFundSimilarList(params));
    }

    /**
     * 公募基金公告
     *
     * @param params 基金id
     * @return 基金公告
     */
    @GetMapping("notice-info")
    public ResultVo<List<ManualFundNoticeInfoVO>> noticeInfo(ManualFundNoticeReq params) {
        return ResultVo.ok(service.noticeList(params));
    }

    /**
     * 公募基金投资类信息说明
     *
     * @param refId 基金id
     * @return 投资类信息说明
     */
    @GetMapping("invest-info")
    public ResultVo<ManualFundInvestInfoVO> investInfo(@RequestParam("refId") String refId) {
        return ResultVo.ok(service.investInfo(refId));
    }

    /**
     * 基金费率信息
     *
     * @param refId 基金id
     * @return 基金费率信息
     */
    @GetMapping("fee")
    public ResultVo<FundBaseFeeVO> fee(String refId) {
        return ResultVo.ok(this.service.getFundFee(refId));
    }

}
