package com.smppw.analysis.domain.dao;

import com.smppw.analysis.domain.dataobject.CompanyFittedCurveDo;
import com.smppw.analysis.domain.dataobject.CompanyFittedCurveWeeklyDo;
import com.smppw.analysis.domain.mapper.core.CompanyFittedCurveDoMapper;
import com.smppw.analysis.domain.mapper.core.CompanyFittedCurveWeeklyDoMapper;
import com.smppw.common.cache.CaffeineLocalCache;
import com.smppw.common.pojo.dto.NavDto;
import com.smppw.common.pojo.enums.Frequency;
import com.smppw.constants.DateConst;
import com.smppw.utils.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Repository
public class CompanyNavDao {

    @Autowired
    private CompanyFittedCurveDoMapper companyFittedCurveDoMapper;

    @Autowired
    private CompanyFittedCurveWeeklyDoMapper companyFittedCurveWeeklyDoMapper;

    public Map<String, List<NavDto>> getNav(List<String> companyIdList, String startDate, String endDate, Integer curveTypeId, Integer strategyId, Map<String, Frequency> secFrequencyMap, boolean needUpdateAndCreateTime) {

        List<String> monthSecIdList = new ArrayList<>();
        List<String> weekSecIdList = new ArrayList<>();
        for (String companyId : companyIdList) {
            Frequency frequency = secFrequencyMap.get(companyId);
            if (Frequency.Weekly == frequency) {
                weekSecIdList.add(companyId);
            } else {
                monthSecIdList.add(companyId);
            }
        }

        Map<String, List<NavDto>> secNavDtoMap = new HashMap<>();

        if (monthSecIdList.size() > 0) {
            String startDateMonth = null;
            if (StringUtils.isNotEmpty(startDate) && startDate.length() >= 7) {
                startDateMonth = startDate.substring(0, 7);
            }
            List<CompanyFittedCurveDo> navDoList = companyFittedCurveDoMapper.listNavByTimeInterval(monthSecIdList, startDateMonth, endDate, curveTypeId, strategyId, needUpdateAndCreateTime);
            Map<String, List<NavDto>> secMonthNavDtoMap = monthNavDoListToNavDtoMap(navDoList);
            secNavDtoMap.putAll(secMonthNavDtoMap);
        }

        if (weekSecIdList.size() > 0) {
            String startYearWeek = null;
            String endYearWeek = null;
            if (StringUtils.isNotEmpty(startDate)) {
                startYearWeek = CaffeineLocalCache.getYearWeekByDate(startDate);
            }
            if (StringUtils.isNotEmpty(endDate)) {
                endYearWeek = CaffeineLocalCache.getYearWeekByDate(endDate);
            }

            List<CompanyFittedCurveWeeklyDo> navDoList = companyFittedCurveWeeklyDoMapper.listNavByTimeInterval(weekSecIdList, startYearWeek, endYearWeek, curveTypeId, strategyId, needUpdateAndCreateTime);
            Map<String, List<NavDto>> secWeekNavDtoMap = weekNavDoListToNavDtoMap(navDoList);
            secNavDtoMap.putAll(secWeekNavDtoMap);
        }

        return secNavDtoMap;
    }

    /**
     *  领域对象集合转为数据传输对象集合
     * @param navDoList 领域对象集合
     * @return  数据传输对象集合  key-基金ID  value-数据传输对象集合
     */
    private Map<String, List<NavDto>> monthNavDoListToNavDtoMap(List<CompanyFittedCurveDo> navDoList) {
        Map<String, List<NavDto>> fundIdNavDtoListMap = new HashMap<>();

        if (navDoList == null) {
            return fundIdNavDtoListMap;
        }

        for (CompanyFittedCurveDo navDo : navDoList) {
            NavDto navDto = new NavDto();
            navDto.setFundId(navDo.getCompanyId());
            if (navDo.getEndDate() == null) {
                continue;
            }
            navDto.setPriceDate(navDo.getEndDate());
            navDto.setOriginalNav(navDo.getCumulativeNav());
            navDto.setCumulativeNav(navDo.getCumulativeNav());
            navDto.setCumulativeNavWithdrawal(navDo.getCumulativeNav());
            if (navDo.getUpdatetime() != null) {
                navDto.setUpdatetime(DateUtil.DateToString(navDo.getUpdatetime(), DateConst.YYYY_MM_DD));
            }
            if (navDo.getCreatetime() != null) {
                navDto.setCreatetime(DateUtil.DateToString(navDo.getCreatetime(), DateConst.YYYY_MM_DD));
            }

            if (fundIdNavDtoListMap.containsKey(navDo.getCompanyId())) {
                List<NavDto> navDtoList = fundIdNavDtoListMap.get(navDo.getCompanyId());
                navDtoList.add(navDto);
            } else {
                List<NavDto> navDtoList = new ArrayList<>();
                navDtoList.add(navDto);
                fundIdNavDtoListMap.put(navDo.getCompanyId(), navDtoList);
            }
        }
        return fundIdNavDtoListMap;
    }

    /**
     *  领域对象集合转为数据传输对象集合
     * @param navDoList 领域对象集合
     * @return  数据传输对象集合  key-基金ID  value-数据传输对象集合
     */
    private Map<String, List<NavDto>> weekNavDoListToNavDtoMap(List<CompanyFittedCurveWeeklyDo> navDoList) {
        Map<String, List<NavDto>> fundIdNavDtoListMap = new HashMap<>();

        if (navDoList == null) {
            return fundIdNavDtoListMap;
        }

        for (CompanyFittedCurveWeeklyDo navDo : navDoList) {
            NavDto navDto = new NavDto();
            navDto.setFundId(navDo.getCompanyId());
            String priceDate = CaffeineLocalCache.getLastTradeDateByYearWeek(navDo.getYearWeek());
            if (StringUtils.isEmpty(priceDate)) {
                continue;
            }
            navDto.setPriceDate(priceDate);
            navDto.setOriginalNav(navDo.getCumulativeNav());
            navDto.setCumulativeNav(navDo.getCumulativeNav());
            navDto.setCumulativeNavWithdrawal(navDo.getCumulativeNav());
            if (navDo.getUpdatetime() != null) {
                navDto.setUpdatetime(DateUtil.DateToString(navDo.getUpdatetime(), DateConst.YYYY_MM_DD));
            }
            if (navDo.getCreatetime() != null) {
                navDto.setCreatetime(DateUtil.DateToString(navDo.getCreatetime(), DateConst.YYYY_MM_DD));
            }

            if (fundIdNavDtoListMap.containsKey(navDo.getCompanyId())) {
                List<NavDto> navDtoList = fundIdNavDtoListMap.get(navDo.getCompanyId());
                navDtoList.add(navDto);
            } else {
                List<NavDto> navDtoList = new ArrayList<>();
                navDtoList.add(navDto);
                fundIdNavDtoListMap.put(navDo.getCompanyId(), navDtoList);
            }
        }
        return fundIdNavDtoListMap;
    }
    
}
