package com.smppw.analysis.domain.manager.info;

import cn.hutool.core.date.DateUtil;
import com.smppw.analysis.domain.dao.FundInformationDao;
import com.smppw.analysis.domain.dataobject.FundInformationDo;
import com.smppw.analysis.domain.dto.info.FundHeadInfoVO;
import com.smppw.analysis.domain.dto.info.HeadInfoParams;
import com.smppw.analysis.domain.service.BaseInfoService;
import com.smppw.analysis.infrastructure.exception.APIException;
import com.smppw.common.pojo.enums.RaiseType;

import java.util.Map;

public abstract class AbstractFundHeadInfo<R extends FundHeadInfoVO> implements HeadInfo<R> {
    protected final BaseInfoService baseInfoService;
    protected final FundInformationDao fundInformationDao;

    public AbstractFundHeadInfo(BaseInfoService baseInfoService, FundInformationDao fundInformationDao) {
        this.baseInfoService = baseInfoService;
        this.fundInformationDao = fundInformationDao;
    }

    protected abstract Map<String, String> getSecNameMap(FundInformationDo fundInfo);

    @Override
    public R get(HeadInfoParams params) {
        FundHeadInfoVO headInfo = new FundHeadInfoVO();
        String fundId = params.getSecId();
        FundInformationDo fundInfo = this.fundInformationDao.getFundInfoById(fundId);
        if (fundInfo == null) {
            throw new APIException("基金不存在");
        }
        if (fundInfo.getRaiseType() == null) {
            throw new APIException("基金的募集类型不能为空");
        }

        String trustId = fundInfo.getTrustId();
        String primaryBenchmarkId = fundInfo.getPrimaryBenchmarkId();
        Map<String, String> secNameMap = this.getSecNameMap(fundInfo);
        headInfo.setRefId(fundId);
        headInfo.setDetailId(fundId);
        headInfo.setRefShortName(fundInfo.getFundShortName());
        headInfo.setRefName(fundInfo.getFundName());
        headInfo.setRegisterNumber(fundInfo.getRegisterNumber());
        headInfo.setAmacUrl(fundInfo.getAmacUrl());
        headInfo.setRaiseType(fundInfo.getRaiseType() == 1 ? "私募" : "公募");
        RaiseType raiseType = RaiseType.getRaiseType((int) fundInfo.getRaiseType());
        headInfo.setRaiseTypeValue(raiseType.name());
        headInfo.setInceptionDate(DateUtil.formatDate(fundInfo.getInceptionDate()));
        headInfo.setTrustId(trustId);
        headInfo.setTrustName(secNameMap.get(trustId));
        headInfo.setManagers(null);
        headInfo.setFundStatusValue(null);
        headInfo.setFundScale(null);
        headInfo.setCompanyAssetSize(null);
        headInfo.setPrimaryBenchmarkId(primaryBenchmarkId);
        headInfo.setPrimaryBenchmarkName(secNameMap.get(primaryBenchmarkId));
        return this.getFund(params, headInfo);
    }

    public abstract R getFund(HeadInfoParams params, FundHeadInfoVO headInfo);
}
