package com.smppw.analysis.domain.manager.performance.handler;

import cn.hutool.core.collection.ListUtil;
import com.smppw.analysis.domain.dto.performance.MulSecMulIndicatorParams;
import com.smppw.analysis.domain.manager.performance.AbstractSingleSecPerformance;
import com.smppw.analysis.domain.manager.performance.PerformanceConstants;
import com.smppw.analysis.domain.service.BaseIndicatorServiceV2;
import com.smppw.analysis.domain.service.BaseInfoService;
import com.smppw.common.pojo.dto.calc.IndicatorCalcPropertyDto;
import com.smppw.common.pojo.dto.indicator.CalcMultipleSecMultipleTimeRangeIndicatorReq;
import com.smppw.common.pojo.enums.DateIntervalType;
import com.smppw.common.pojo.enums.Indicator;
import com.smppw.constants.Consts;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * @author mozuwen
 * @date 2023/8/9 16:00
 * @description 多标的多指标的的实现
 */
@Component(PerformanceConstants.MUL_SEC_MUL_INDICATOR)
public class MulSecMulIndicatorHandler extends AbstractSingleSecPerformance<MulSecMulIndicatorParams, Map<String, List<IndicatorCalcPropertyDto>>> {

    public MulSecMulIndicatorHandler(BaseInfoService baseInfoService, BaseIndicatorServiceV2 baseIndicatorServiceV2) {
        super(baseInfoService, baseIndicatorServiceV2);
    }

    @Override
    protected CalcMultipleSecMultipleTimeRangeIndicatorReq buildCalcReq(MulSecMulIndicatorParams params, DateIntervalType dateIntervalType) {
        CalcMultipleSecMultipleTimeRangeIndicatorReq req = super.buildCalcReq(params, dateIntervalType);
        req.setRiskOfFreeId(Consts.RISK_OF_FREE);
        List<Indicator> indicatorList = params.getIndicatorList();
        req.setIndicatorList(indicatorList);
        req.setGeoExtraindicatorList(ListUtil.toList());
        req.setRiskOfFreeId(Consts.RISK_OF_FREE);
        req.setIndexIdList(ListUtil.toList(params.getBenchmarkId()));
        return req;
    }

    @Override
    protected Map<String, List<IndicatorCalcPropertyDto>> bizSecHandle(MulSecMulIndicatorParams params) {
        CalcMultipleSecMultipleTimeRangeIndicatorReq req = this.buildCalcReq(params, DateIntervalType.DefaultInterval);
        Map<String, List<IndicatorCalcPropertyDto>> indicatorMap = this.baseIndicatorServiceV2.calcMultipleSecMultipleTimeRangeIndicator(req);
        return indicatorMap;
    }

}
