package com.smppw.analysis.domain.mapper.core;

import com.smppw.analysis.domain.dataobject.FundFeeDo;
import com.smppw.analysis.domain.dataobject.FundInformationDo;
import com.smppw.analysis.domain.dataobject.FundSimilarDo;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

@Repository
public interface FundInformationDoMapper {
    List<FundInformationDo> listFundInfo(@Param("fundIdList") List<String> fundIdList);

    /**
     * 获取基金经理的主要投资策略种类和为主的投资策略
     *
     * @param managerId 基金经理id
     * @return 基金经理的主要投资策略种类和为主的投资策略
     */
    List<Map<String, Object>> getManagerFundFeature(@Param("managerId") String managerId);

    /**
     * 获取市场基金的名字
     *
     * @param fundId 基金id
     * @return 市场基金的名字
     */
    FundInformationDo getFundByFundId(@Param("fundId") String fundId);

    /**
     * 基金业绩排名
     *
     * @param params /
     * @return /
     */
    List<Map<String, Object>> getFundRankByCall(Map<String, Object> params);

    /**
     * 获取同公司末级策略相同的所有产品中最近一年有净值的产品
     *
     * @param params /
     * @return /
     */
    List<FundSimilarDo> getSameCompanyFundIds(Map<String, Object> params);

    /**
     * 取产品末级策略同策略近一年收益排名前50的产品（排除本产品
     *
     * @param params /
     * @return /
     */
    List<FundSimilarDo> getSameStrategyFundIds(Map<String, Object> params);

    /**
     * 获取基金的申赎费率信息
     * @param fundId 基金id
     * @return 基金的申赎费率信息
     */
    FundFeeDo getFundFee(String fundId);

}