package com.smppw.analysis.infrastructure.config;

import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.SqlSessionTemplate;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.jdbc.DataSourceBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;

import javax.sql.DataSource;

@Configuration
@MapperScan(basePackages = {"com.smppw.analysis.domain.mapper.mf"}, sqlSessionFactoryRef = "mfSqlSessionFactory")
public class MfDataSourceConfig {
    @Bean("mfDataSource")
    @ConfigurationProperties(prefix = "spring.datasource.mf")
    public DataSource mfDataSource() {
        return DataSourceBuilder.create().build();
    }

    @Bean(name = "mfSqlSessionFactory")
    public SqlSessionFactory mfSqlSessionFactory(@Qualifier("mfDataSource") DataSource datasource)
            throws Exception {
        SqlSessionFactoryBean bean = new SqlSessionFactoryBean();
        bean.setDataSource(datasource);
        bean.setMapperLocations(new PathMatchingResourcePatternResolver().getResources("classpath*:mapping/mf/*.xml"));
        return bean.getObject();// 设置mybatis的xml所在位置
    }

    @Bean("mfSqlSessionTemplate")
    public SqlSessionTemplate mfSqlSessionTemplate(
            @Qualifier("mfSqlSessionFactory") SqlSessionFactory sessionFactory) {
        return new SqlSessionTemplate(sessionFactory);
    }
}
